/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.mega.gasp.platform.impl;

import org.mega.gasp.platform.Session;

/**
 * Session represent the connection of the user in the application.
 * So it is instantiate immedialtly after the login of the user.
 * The Session ID is used for identification of the user on a request.
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */


public class SessionImpl implements Session {

	private int sessionID;
	private int actorID;
	private int actorSessionID;
	private String pseudo;
	private long timeout;

	
	public SessionImpl(int aID, long _timeout){ 
		actorID = aID; 
		sessionID = PlatformImpl.getPlatform().getIDManager().generateSID();
		actorSessionID = 0;
		timeout = _timeout;
	}


	/**
	 * Returns the Session ID of the Session object.
	 * 
	 * @return the Session ID
	 */
	public int getSessionID() {
		return sessionID;
	}

	
	/**
	 * Returns the Actor ID of the user.
	 * 
	 * @return the Actor ID
	 */
	public int getActorID() {
		return actorID;
	}


	/**
	 * Returns the ActorSession ID if the user have already choiced a game session.
	 * 
	 * @return the ActorSession ID
	 */
	public int getActorSessionID() {
		return actorSessionID;
	}

	
	/**
	 * Returns the current pseudo name of the user session.
	 * 
	 * @return the ActorSession ID
	 */
	public String getPseudoName() {
		return pseudo;
	}
	
	
	/**
	 * Sets the ActorSession ID after game session choose by the user.
	 * @param aSID the ActorSession ID
	 */
	public void setActorSessionID(int aSID) {
		actorSessionID = aSID;
	}
	
	
	/**
	 * Set the current pseudo name of the user session.
	 */
	public void setPseudoName(String name) {
		pseudo = name;
	}
	
	 /**
     * Returns last time infos received for the user associated with the actorSession.
     * 
     * @return long the application timeout
     */
    public long getTimeout() {
        return timeout;
    }


    /**
     * Save last time infos received for the user associated with the actorSession.
     */
    public void setTimeout(long _timeout) {
        timeout = _timeout;
    }
    
    
    /**
     * Save last time infos received for the user associated with the actorSession.
     */
    public void refreshTimeout() {
        setTimeout(System.currentTimeMillis());
    }

}
