/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.mega.gasp.platform.servlets;

import org.apache.log4j.Category;
import org.mega.gasp.platform.impl.PlatformImpl;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.PrintWriter;


import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This servlet is call the first time the user log in with the Application.
 * The HTTP parameters are :
 * - uid the UserID of the operator
 * - appID the ApplicationID hard coded in the application.
 * - username the choosed username
 * - password the choosed password
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class FirstLogin extends HttpServlet {

    private PlatformImpl platform;
    private Category cat;

    public void init(){
        ServletContext app = getServletContext();
        platform = PlatformImpl.getPlatform(app.getInitParameter("gasp.home"));
        cat = Category.getInstance("FirstLoginServlet");
    }
    
	public void doGet(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
	    try{
		    // get http request parameters for login 
			String uid = request.getParameter("uid");
			short appID = Short.parseShort(((String) request.getParameter("appID")));
			String username = request.getParameter("username");
			String password = request.getParameter("password");
			
			//debug
			cat.debug("uid: "+uid);
			cat.debug("appID: "+appID);
			cat.debug("username: "+username);
			cat.debug("password: "+password);
				
			// registration of the user with his choice of username and password
			// return a short aID, permanent actorID associated with the userID and applicationID
			// or the short "0" if the user have not the appropriate rights
			int aID = platform.getDBManager().registerNewActor(uid,appID,username,password);
			
			response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
			PrintWriter out = new PrintWriter(response.getOutputStream());
			out.println("aID="+aID);
			
			cat.debug("aID sended to Flash: " + aID);
			out.close();
	    }
	    catch (Exception e){
	        cat.info("cannot complied to received first login request");
	    }
	    finally{

	    }
		
	}
	
	
	public void doPost(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
	    DataOutputStream dos = null;
	    try{
		    // get http request parameters for login 
			String uid = request.getParameter("uid");
			short appID = Short.parseShort(((String) request.getParameter("appID")));
			String username = request.getParameter("username");
			String password = request.getParameter("password");
			
			// registration of the user with his choice of username and password
			// return a short aID, permanent actorID associated with the userID and applicationID
			// or the short "0" if the user have not the appropriate rights
			int aID = platform.getDBManager().registerNewActor(uid,appID,username,password);
			
			// write the short aID in the http response
			dos = new DataOutputStream(response.getOutputStream());
		
			dos.writeShort(Short.parseShort(aID+""));
	    }
	    catch (Exception e){
	        cat.info("cannot complied to received first login request");
	    }
	    finally{
	        if (dos!=null) try{dos.close();} catch (Exception e){}
	    }
		
	}
}
