/***
 * GASP: a gaming services platform for mobile multiplayer games.
 * Copyright (C) 2004 CNAM/INT
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: gasp-team@objectweb.org
 *
 * Author: Romain Pellerin
 */
package org.mega.gasp.platform.servlets;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.*;

import org.apache.log4j.Category;
import org.mega.gasp.platform.impl.PlatformImpl;
import org.mega.gasp.platform.utils.ApplicationInstanceInfos;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Vector;

/**
 * This servlet provide to get Lobby informations about current ApplicationInstance
 * of the Application.
 * The HTTP parameter is :
 * - sID the Session ID of the user
 * 
 * @author PELLERIN Romain (pellerin@cnam.fr) - MEGA Project 
 */
public class Lobby extends HttpServlet {
	
	private Category cat;
    private PlatformImpl platform;

    public void init(){
        ServletContext app = getServletContext();
        platform = PlatformImpl.getPlatform(app.getInitParameter("gasp.home"));
        cat = Category.getInstance("LobbyServlet");
    }
	
    
    public void doGet(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
    	String sID = request.getParameter("sID");

		// requiring the lobby infos to the platform for the application
		// associated with the sID
		Vector lobbyInfos = platform.getLobby().getApplicationInstancesInfos(
				Integer.parseInt(sID));

		// encoded string to be sent to Flash clients
		String data = "";
			
		// push in the DataOutputStream the number of application instances
		data += addEncodingSpacer(""+lobbyInfos.size());

		for (Iterator iter = lobbyInfos.iterator(); iter.hasNext();) {
			ApplicationInstanceInfos appInstanceInfos = (ApplicationInstanceInfos) iter
					.next();

			// push the aIID of the current application instance
			data += addEncodingSpacer(""+appInstanceInfos.getApplicationInstanceID());

			// push the minimum starting number of actors
			data += addEncodingSpacer(""+appInstanceInfos.getMinActors());

			// push the number of actors currently connected in
			data += addEncodingSpacer(""+appInstanceInfos.getNbPlayers());

			// push the maximum number of actors
			data += addEncodingSpacer(""+appInstanceInfos.getMaxActors());

			for (Iterator iter2 = appInstanceInfos.getPlayerList().iterator(); iter2
					.hasNext();) {
				String username = (String) iter2.next();
				Integer actorID = (Integer) iter2.next();

				// push the username and the actor ID of the current actor
				data += addEncodingSpacer(username);
				data += addEncodingSpacer(""+actorID.intValue());
			}
		}
		
		// send the lobby encoded string to Flash client
		response.setContentType("application/x-www-urlform-encoded; charset=UTF-8");
		PrintWriter out = new PrintWriter(response.getOutputStream());
		out.println("data=" + data);

		cat.debug("lobby sended to Flash: " + data);
		out.close();
    }
	
	
	public void doPost(HttpServletRequest request,HttpServletResponse response) throws IOException, ServletException {
	    DataOutputStream dos = null;
	    OutputStream out = null;
	    
	    try{
		    String sID = request.getParameter("sID");
			
			// requiring the lobby infos to the platform for the application associated with the sID
			Vector lobbyInfos = platform.getLobby().getApplicationInstancesInfos(Integer.parseInt(sID));
			
			out = response.getOutputStream();
			dos = new DataOutputStream(out);
			
			// push in the DataOutputStream the number of application instances
			dos.writeShort(lobbyInfos.size());
			
			for (Iterator iter = lobbyInfos.iterator(); iter.hasNext();){
				ApplicationInstanceInfos appInstanceInfos = (ApplicationInstanceInfos) iter.next();
				
				// push the aIID of the current application instance
				dos.writeShort(appInstanceInfos.getApplicationInstanceID());
				
				// push the minimum starting number of actors
				dos.writeShort(appInstanceInfos.getMinActors());
				
				
				// push the number of actors currently connected in
				dos.writeShort(appInstanceInfos.getNbPlayers());
				
				// push the maximum number of actors 
				dos.writeShort(appInstanceInfos.getMaxActors());
				
				for (Iterator iter2 = appInstanceInfos.getPlayerList().iterator(); iter2.hasNext();){
					String username = (String) iter2.next();
					Integer actorID = (Integer) iter2.next();
					
					// push the username and the actor ID of the current actor
					dos.writeUTF(username);
					dos.writeShort(actorID.intValue());
				}
			}
	    }
	    catch (Exception e){
	        cat.info("cannot comply to received lobby request");
	    }
	    finally{
	        if (dos!=null) try{ dos.close();} catch (Exception e){}
	        if (out!=null) try{ out.close();} catch (Exception e){}
	    }
	}
	
	
	// for encoded string communication only
	private int indexTreatment = 0;
	
	private String addEncodingSpacer(Object o) {
		 return o+"|";
	}
	
	// for encoded string communication only
	private String getNextParam(String code) {
		 String param = "";
		 char c = code.charAt(indexTreatment);

		 while ((indexTreatment < code.length()) && c != '|' ){
			 param += c;
			 indexTreatment++;
			 if (indexTreatment < code.length()) c = code.charAt(indexTreatment);
			 else break;
		 }
		 if (indexTreatment < code.length()) indexTreatment++; // "pointeur" apres |

		 return param;
	 }
}
